@@ -10,7 +10,7 @@
     :license: BSD, see LICENSE for more details.
 """
 
-nodeenv_version = '0.9.2'
+nodeenv_version = '0.9.3'
 
 import sys
 import os
@@ -34,6 +34,19 @@ abspath = os.path.abspath
 # ---------------------------------------------------------
 # Utils
 
+def clear_output(out):
+    """
+    Remove new-lines and
+    """
+    return out.decode('utf-8').replace('\n', '')
+
+
+def remove_env_bin_from_path(env, env_bin_dir):
+    """
+    Remove bin directory of the current environment from PATH
+    """
+    return env.replace(env_bin_dir + ':', '')
+
 
 def node_version_from_opt(opt):
     """
@@ -42,7 +55,7 @@ def node_version_from_opt(opt):
     if opt.node == 'system':
         out, err = subprocess.Popen(
             ["node", "--version"], stdout=subprocess.PIPE).communicate()
-        return parse_version(out.replace('\n', '').replace('v',  ''))
+        return parse_version(clear_output(out).replace('v', ''))
 
     return parse_version(opt.node)
 
@@ -381,7 +394,7 @@ def copy_node_from_prebuilt(env_dir, src_dir):
     Copy prebuilt binaries into environment
     """
     logger.info('.', extra=dict(continued=True))
-    callit(['cp', '-r', src_dir + '/node-v*/*', env_dir], True, env_dir)
+    callit(['cp', '-a', src_dir + '/node-v*/*', env_dir], True, env_dir)
     logger.info('.', extra=dict(continued=True))
 
 
@@ -524,8 +537,11 @@ def install_activate(env_dir, opt):
 
     shim_node = join(bin_dir, "node")
     if opt.node == "system":
-        _, which_node_output = callit(['which', 'node'])
-        shim_node = which_node_output[0]
+        env = os.environ.copy()
+        env.update({'PATH': remove_env_bin_from_path(env['PATH'], bin_dir)})
+        which_node_output, _ = subprocess.Popen(
+            ["which", "node"], stdout=subprocess.PIPE, env=env).communicate()
+        shim_node = clear_output(which_node_output)
 
     for name, content in files.items():
         file_path = join(bin_dir, name)
@@ -535,7 +551,14 @@ def install_activate(env_dir, opt):
         content = content.replace('__SHIM_NODE__', shim_node)
         content = content.replace('__BIN_NAME__', os.path.basename(bin_dir))
         content = content.replace('__MOD_NAME__', mod_dir)
-        writefile(file_path, content, append=opt.python_virtualenv)
+        # if we call in the same environment:
+        #   $ nodeenv -p --prebuilt
+        #   $ nodeenv -p --node=system
+        # we should get `bin/node` not as binary+string.
+        # `bin/activate` should be appended if we inside
+        # existing python's virtual environment
+        need_append = False if name in ('node', 'shim') else opt.python_virtualenv
+        writefile(file_path, content, append=need_append)
         os.chmod(file_path, mode_0755)
 
 
